﻿using KelownaSoftware.Application.Common.Constants;
using KelownaSoftware.Application.Common.Exceptions;
using KelownaSoftware.Application.Common.Interfaces;
using MediatR.Pipeline;

namespace KelownaSoftware.Application.Common.Behaviours
{
    public abstract class UserContextAuthorizationRequestPreprocessorBase<IRequest, T> : IRequestPreProcessor<IRequest> where T : Enum
    {
        private readonly IUserContext _userContext;

        protected UserContextAuthorizationRequestPreprocessorBase(IUserContext userContext)
        {
            _userContext = userContext;
        }

        public virtual Task Process(IRequest request, CancellationToken cancellationToken)
        {
            return Process(request, cancellationToken, null);
        }

        public virtual async Task Process(IRequest request, CancellationToken cancellationToken, params T[] allowedClaimTypes)
        {
            foreach (var claim in _userContext.User.Claims)
            {
                if (claim.Value == Enum.GetName(typeof(T), ApplicationCommonConstants.ALL_PERMISSIONS_VALUE))
                {
                    return;
                }

                if (allowedClaimTypes != null)
                {
                    foreach (var claimType in allowedClaimTypes)
                    {
                        if (claim.Value == Enum.GetName(typeof(T), claimType))
                        {
                            return;
                        }
                    }
                }
            }

            throw new NotAuthorizedException(request.GetType().Name, _userContext.UserId);
        }
    }
}
