﻿using KelownaSoftware.Application.Common.Interfaces;

namespace KelownaSoftware.Application.Common.Notifications.Email
{
    public abstract class EmailNotificationBase : IEmailNotification
    {
        private List<string> _to;
        public IEnumerable<string> To { get => _to; }
        private List<string> _cc;
        public IEnumerable<string> CC { get => _cc; }
        private List<string> _bcc;
        public IEnumerable<string> BCC { get => _bcc; }

        private List<IEmailAttachment> _attachments;
        public IEnumerable<IEmailAttachment> Attachments { get => _attachments; }
        public virtual string Subject { get; set; }
        public virtual string HtmlContent { get; set; }
        public virtual string PlainTextContent { get; set; }

        public EmailNotificationBase()
        {
            _to = new List<string>();
            _cc = new List<string>();
            _bcc = new List<string>();
            _attachments = new List<IEmailAttachment>();
        }

        public void AddTo(string email)
        {
            _to.Add(email);
        }

        public void AddTos(IEnumerable<string> tos)
        {
            _to.AddRange(tos);
        }

        public void AddCC(string email)
        {
            _cc.Add(email);
        }

        public void AddCCs(IEnumerable<string> ccs)
        {
            _cc.AddRange(ccs);
        }

        public void AddBCC(string email)
        {
            _bcc.Add(email);
        }

        public void AddBCCs(IEnumerable<string> bccs)
        {
            _bcc.AddRange(bccs);
        }

        public void AddAttachment(EmailAttachment emailAttachment)
        {
            if (emailAttachment != null)
            {
                _attachments.Add(emailAttachment);
            }
        }



    }
}
