﻿using KelownaSoftware.Application.Common.Interfaces;
using KelownaSoftware.Application.Common.Notifications.Email;
using KelownaSoftware.Application.Common.Utilities;
using MediatR.Pipeline;
using Microsoft.AspNetCore.Http;

namespace KelownaSoftware.Application.AggregateRoots.Authentication.ConfirmEmailAddress
{
    public class ConfirmEmailAddressPostProcessor : IRequestPostProcessor<ConfirmEmailAddressCommand, ConfirmEmailAddressResult>
    {
        private readonly IEmailService _emailService;
        private readonly IHttpContextAccessor _httpContextAccessor;

        public ConfirmEmailAddressPostProcessor(IEmailService emailService, IHttpContextAccessor httpContextAccessor)
        {
            _emailService = emailService;
            _httpContextAccessor = httpContextAccessor;
        }

        public async Task Process(ConfirmEmailAddressCommand request, ConfirmEmailAddressResult response, CancellationToken cancellationToken)
        {
            var baseUrl = HttpContextHelpers.GetBaseUrl(_httpContextAccessor);
            var emailConfirmed = new EmailConfirmedEmailNotification(baseUrl);

            var emailAddress = response.GetEmailAddresses();
            emailConfirmed.AddTo(emailAddress);

            await _emailService.SendAsync(emailConfirmed, cancellationToken);

        }
    }
}
