import { Injectable } from '@angular/core';
import { BaseResponseViewModel, ValidationErrorViewModel } from '@application/shared';
import { ToastrService } from 'ngx-toastr';

@Injectable({ providedIn: 'root' })
export class NotificationService {

  private toasterOptions: any;

  constructor(private toastrService: ToastrService) {
    toastrService.toastrConfig.progressBar = true;
    this.toasterOptions = {
      titleClass: "toast-title",
      messageClass: "toast-message"
    };
  }

  public ShowError(error: string | BaseResponseViewModel, title: string = "") {
    let errorMessage = '';

    if (error instanceof Object) {
      if (error.validationErrors?.length > 0) {
        errorMessage = error.validationErrors.map(x => ValidationErrorViewModel.ToString(x)).join('\r\n');
      } else {
        errorMessage = error.error;
      }
    } else {
      errorMessage = error;
    }

    this.toastrService.error(errorMessage, title, this.toasterOptions);
  }

  public ShowInfo(text: string, title: string = "") {
    this.toastrService.info(text, title, this.toasterOptions);
  }

  public ShowSuccess(text: string, title: string = "") {
    this.toastrService.success(text, title, this.toasterOptions);
  }

  public ShowWarning(text: string, title: string = "") {
    this.toastrService.warning(text, title, this.toasterOptions);
  }
}
