import { Injectable } from '@angular/core';
import { Title } from '@angular/platform-browser';
import { Router } from '@angular/router';
import { Observable, ReplaySubject } from 'rxjs';

@Injectable({
  providedIn: 'root'
})
export class PageTitleService {

  private _defaultPageTitle: string = "Untitled Page";
  private _pageTitle: ReplaySubject<string> = new ReplaySubject<string>();
  private _currentEntityRouteSegments: string[] | null = null;

  public getPageTitle(): Observable<string> {
    return this._pageTitle.asObservable();
  }

  public setPageTitle(value: string) {
    if (value && value.length > 0) {
      this._pageTitle.next(value);
    }

    // if current route is not a subset of the stored entity route, then overwrite Tab title with the actual page title
    // otherwise leave it as is b/c we're still showing the entity the Tab title describes
    //if (!RouteHelpers.isRouteWithinRoot(this.router, this._currentEntityRouteSegments)) {
    //  this._currentEntityRouteSegments = null;
    this.titleService.setTitle(value + " | Kelowna Software");
    //}
  }

  constructor(
    private titleService: Title,
    public router: Router)
  {
    this.setPageTitle(this._defaultPageTitle);
  }
}
