import { Injectable, OnDestroy } from '@angular/core';
import { GetCurrentUserViewModel } from '@application/users';
import { AccountService } from '@infrastructure/services/account.service';
import { ApiService } from '@infrastructure/services/api.service';
import { ReplaySubject, Subject, take, takeUntil } from 'rxjs';

@Injectable({
  providedIn: 'root'
})
export class CurrentUserService implements OnDestroy{

  public currentUser: ReplaySubject<GetCurrentUserViewModel> = new ReplaySubject<GetCurrentUserViewModel>();
  public destroy$: Subject<boolean> = new Subject();

  constructor(
    private apiService: ApiService,
    accountService: AccountService,
  ) {

    accountService.userSignedIn
      .pipe(
        takeUntil(this.destroy$)
      )
      .subscribe(() => {
        this.getCurrentUser();
      })
  }

  private getCurrentUser(): void {
    this.apiService.get<GetCurrentUserViewModel>('api/users/current')
      .pipe(
        take(1)
      )
      .subscribe((result: GetCurrentUserViewModel) => {
        this.currentUser.next(result);
      });
  }

  ngOnDestroy(): void {
    this.destroy$.next(true);
    this.destroy$.unsubscribe();
  }

}
