import { Component, OnInit } from '@angular/core';
import { FormGroup, FormBuilder, Validators } from '@angular/forms';
import { Router, ActivatedRoute } from '@angular/router';
import { SignInCommand } from '@application/account';
import { AccountService } from '@infrastructure/services';
import { take, finalize } from 'rxjs';

@Component({
  selector: 'app-sign-in',
  templateUrl: './sign-in.component.html',
  styleUrls: ['./sign-in.component.css']
})
export class SignInComponent implements OnInit{
  public error = '';
  public loading: boolean = false;
  public submitted: boolean = false;
  private returnUrl!: string;

  loginForm!: FormGroup;
  get f() { return this.loginForm.controls; }

  constructor(
    private formBuilder: FormBuilder,
    private router: Router,
    private route: ActivatedRoute,
    private accountService: AccountService
  ) {
    

  }

  ngOnInit() {
    this.loginForm = this.formBuilder.group({
      username: ['', [Validators.required, Validators.email, Validators.minLength(3)]],
      password: ['', [Validators.required, Validators.minLength(8)]]
    });


    // get return url from route parameters or default to '/'
    this.returnUrl = this.route.snapshot.queryParams['returnUrl'] || '/';
  }

  public onSubmit() {



    this.submitted = true;

    // stop here if form is invalid
    if (this.loginForm.invalid) {
      //return;
    }

    this.loading = true;

    const command = new SignInCommand(this.f['username'].value, this.f['password'].value);

    this.accountService.signin(command)
      .pipe(
        take(1),
        finalize(() => { this.loading = false; })
      )
      .subscribe(result => {
        if (result.error) {
          this.error = result.error;
        }
        else {
          if (result.forcePasswordReset === true && result.passwordResetToken && result.encodedEmail) {
            //the url query params must be and are intentionally lowercase
            const url = `account/forgot-password-confirmation?token=${result.passwordResetToken}&encodedemail=${result.encodedEmail}`;
            this.router.navigateByUrl(url);
          }
          else {
            this.router.navigateByUrl(this.returnUrl);
          }
        }
      })

  }
}
