﻿namespace KelownaSoftware.Domain.Common.Entities.Countries
{
    public class CountryValueObject : BaseEntity
    {
        private readonly CountryValidator _validator;

        private string _name;
        public string Name
        {
            get => _name;
            set
            {
                var temp = value?.Trim() ?? string.Empty;
                _validator.ValidatePropertyValueAndThrow(x => x.Name, temp);
                _name = temp;
            }
        }

        private string _abbreviation;
        public string Abbreviation
        {
            get => _abbreviation;
            set
            {
                var temp = value?.Trim() ?? string.Empty;
                _validator.ValidatePropertyValueAndThrow(x => x.Abbreviation, temp);
                _abbreviation = temp;
            }
        }

        public CountryValueObject() : base()
        {
            _validator = new CountryValidator();
        }

        public CountryValueObject(Country country) : this()
        {
            Id = country.Id;
            Name = country.Name;
            Abbreviation = country.Abbreviation;
        }

        public CountryValueObject(CountryValueObject country) : this()
        {
            Id = country.Id;
            Name = country.Name;
            Abbreviation = country.Abbreviation;
        }

        protected static bool EqualOperator(CountryValueObject left, CountryValueObject right)
        {
            if (ReferenceEquals(left, null) ^ ReferenceEquals(right, null))
            {
                return false;
            }
            return ReferenceEquals(left, null) || left.Equals(right);
        }

        protected static bool NotEqualOperator(CountryValueObject left, CountryValueObject right)
        {
            return !EqualOperator(left, right);
        }

        public override bool Equals(object obj)
        {
            if (obj == null || obj.GetType() != GetType())
            {
                return false;
            }

            var other = (CountryValueObject)obj;

            return GetEqualityComponents().SequenceEqual(other.GetEqualityComponents());
        }

        public override int GetHashCode()
        {
            return GetEqualityComponents()
                .Select(x => x != null ? x.GetHashCode() : 0)
                .Aggregate((x, y) => x ^ y);
        }

        protected IEnumerable<object> GetEqualityComponents()
        {
            yield return Name;
            yield return Abbreviation;
            yield return Id;
        }
    }
}
