﻿using KelownaSoftware.Domain.Common.Entities.Countries;

namespace KelownaSoftware.Domain.Common.Entities.Regions
{
    public class Region : RegionValueObject
    {
        private readonly RegionValidator _validator;

        public virtual Country Country { get; set; }

        private string _countryId;
        public string CountryId
        {
            get => _countryId;
            set
            {
                var temp = value?.Trim();
                _validator.ValidatePropertyValueAndThrow(x => x.CountryId, temp);
                _countryId = temp;
            }
        }

        public Region() : base()
        {
            _validator = new RegionValidator();
        }

        public void SetValues(Region region)
        {
            if (region != null)
            {
                Id = region.Id;
                Name = region.Name;
                Abbreviation = region.Abbreviation;
                CountryId = region.CountryId;
            }
        }

        protected static bool EqualOperator(Region left, Region right)
        {
            if (ReferenceEquals(left, null) ^ ReferenceEquals(right, null))
            {
                return false;
            }
            return ReferenceEquals(left, null) || left.Equals(right);
        }

        protected static bool NotEqualOperator(Region left, Region right)
        {
            return !EqualOperator(left, right);
        }

        public override bool Equals(object obj)
        {
            if (obj == null || obj.GetType() != GetType())
            {
                return false;
            }

            var other = (Region)obj;

            return GetEqualityComponents().SequenceEqual(other.GetEqualityComponents());
        }

        public override int GetHashCode()
        {
            return GetEqualityComponents()
                .Select(x => x != null ? x.GetHashCode() : 0)
                .Aggregate((x, y) => x ^ y);
        }

        protected IEnumerable<object> GetEqualityComponents()
        {
            yield return Name;
            yield return Abbreviation;
            yield return Id;
        }
    }
}
