﻿namespace KelownaSoftware.Domain.Common.Entities.Regions
{
    public class RegionValueObject : BaseEntity
    {
        private readonly RegionValueObjectValidator _validator;

        private string _name;
        public string Name
        {
            get => _name;
            set
            {
                var temp = value?.Trim() ?? string.Empty;
                _validator.ValidatePropertyValueAndThrow(x => x.Name, temp);
                _name = temp;
            }
        }

        private string _abbreviation;
        public string Abbreviation
        {
            get => _abbreviation;
            set
            {
                var temp = value?.Trim() ?? string.Empty;
                _validator.ValidatePropertyValueAndThrow(x => x.Abbreviation, temp);
                _abbreviation = temp;
            }
        }

        public RegionValueObject() : base()
        {
            _validator = new RegionValueObjectValidator();
        }

        public RegionValueObject(RegionValueObject region) : this()
        {
            Id = region.Id;
            Name = region.Name;
            Abbreviation = region.Abbreviation;

        }

        protected static bool EqualOperator(RegionValueObject left, RegionValueObject right)
        {
            if (ReferenceEquals(left, null) ^ ReferenceEquals(right, null))
            {
                return false;
            }
            return ReferenceEquals(left, null) || left.Equals(right);
        }

        protected static bool NotEqualOperator(RegionValueObject left, RegionValueObject right)
        {
            return !EqualOperator(left, right);
        }

        public override bool Equals(object obj)
        {
            if (obj == null || obj.GetType() != GetType())
            {
                return false;
            }

            var other = (RegionValueObject)obj;

            return GetEqualityComponents().SequenceEqual(other.GetEqualityComponents());
        }

        public override int GetHashCode()
        {
            return GetEqualityComponents()
                .Select(x => x != null ? x.GetHashCode() : 0)
                .Aggregate((x, y) => x ^ y);
        }

        protected IEnumerable<object> GetEqualityComponents()
        {
            yield return Name;
            yield return Abbreviation;
            yield return Id;
        }
    }
}
