﻿using System;

namespace KelownaSoftware.Domain.Common.Extensions
{
    public static class LongExtensions
    {
        //don't need anything higher than GB as the max byte[] size in .NET is 
        // int.MaxLength - 56 ~ 2GB (1.9999999469146132469177246094 GB);

        public static string SizeString(this long value, int decimalPlaces = 2)
        {
            if (decimalPlaces < 0)
            {
                throw new ArgumentOutOfRangeException("decimalPlaces");
            }

            const long kb = 1024;
            const long mb = 1024 * kb;
            const long gb = 1024 * mb;
            const long tb = 1024 * gb;

            string sizeWithUnits = string.Empty;

            switch (value)
            {
                case < kb:
                    sizeWithUnits = $"{value:N0} bytes";
                    break;
                case < mb:
                    sizeWithUnits = FormatSize((double)value / kb, decimalPlaces, "KB");
                    break;
                case < gb:
                    sizeWithUnits = FormatSize((double)value / mb, decimalPlaces, "MB");
                    break;
                case < tb:
                    sizeWithUnits = FormatSize((double)value / gb, decimalPlaces, "GB");
                    break;
                default:
                    sizeWithUnits = FormatSize((double)value / tb, decimalPlaces, "TB");
                    break;
            };

            return sizeWithUnits;
        }

        private static string FormatSize(double size, int decimalPlaces, string suffix)
        {
            string sizeString = size.ToString($"N{decimalPlaces}");
            return $"{sizeString} {suffix}";
        }
    }
}
