﻿using System;
using System.Collections.Generic;

namespace KelownaSoftware.Domain.Common.Helpers
{
    public static class ObjectHelpers
    {
        public static bool IsDefaultValue(object valueToCompare, Type type)
        {
            bool output;

            if (Nullable.GetUnderlyingType(type) != null)
            {
                //it's nullable

                output = valueToCompare == null;
            }
            else
            {
                if (type == typeof(string))
                {
                    output = valueToCompare == null || valueToCompare.ToString() == string.Empty;
                }
                else if (type == typeof(string[]))
                {
                    output = valueToCompare == null;
                }
                else if (type == typeof(byte[]))
                {
                    output = valueToCompare == null;
                }
                else if (type == typeof(bool[]))
                {
                    output = valueToCompare == null;
                }
                else if (type == typeof(int[]))
                {
                    output = valueToCompare == null;
                }
                else if (type == typeof(decimal[]))
                {
                    output = valueToCompare == null;
                }
                else
                {
                    var instance = Activator.CreateInstance(type);
                    output = instance.Equals(valueToCompare);
                }

            }

            return output;
        }

        public static string ConvertObjectToString(object value)
        {
            var result = string.Empty;

            if (value != null)
            {
                var valueType = value.GetType();

                if (valueType.IsArray)
                {
                    result = ConvertArrayToCSVString(value, valueType);
                }
                else
                {
                    result = value.ToString();
                }
            }

            return result;
        }

        private static string ConvertArrayToCSVString(object value, Type valueType)
        {
            string result;
            var elementType = valueType.GetElementType();

            if (typeof(string).IsAssignableFrom(elementType))
            {
                result = string.Join(',', value as IEnumerable<string>);
            }
            else if (typeof(bool).IsAssignableFrom(elementType))
            {
                result = string.Join(',', value as IEnumerable<bool>);
            }
            else if (typeof(byte).IsAssignableFrom(elementType))
            {
                result = string.Join(',', value as IEnumerable<byte>);
            }
            else if (typeof(int).IsAssignableFrom(elementType))
            {
                result = string.Join(',', value as IEnumerable<int>);
            }
            else if (typeof(decimal).IsAssignableFrom(elementType))
            {
                result = string.Join(',', value as IEnumerable<decimal>);
            }
            else
            {
                result = string.Empty;
            }

            return result;
        }

    }
}
