﻿namespace KelownaSoftware.Domain.Common.ValueObjects.EmailAddress
{
    public class EmailAddress : ValueObject
    {
        public string Local { get; private set; }
        public string Domain { get; private set; }

        public EmailAddress()
        {

        }

        public EmailAddress(string emailAddress)
        {
            //the email address is not required and could be empty. 
            if (string.IsNullOrWhiteSpace(emailAddress) == false)
            {
                Validate(emailAddress);

                var split = emailAddress.Split('@');
                Local = split[0].Trim();
                Domain = split[1].Trim();
            }
            else
            {
                Local = string.Empty;
                Domain = string.Empty;
            }
        }

        private void Validate(string value)
        {
            _ = new System.Net.Mail.MailAddress(value);
        }

        public static bool IsValid(string emailAddress)
        {
            var output = false;

            if (string.IsNullOrWhiteSpace(emailAddress) == false)
            {
                try
                {
                    new EmailAddress(emailAddress);
                    output = true;
                }
                catch { }
            }

            return output;
        }

        public override string ToString()
        {
            if (IsEmpty)
            {
                return string.Empty;
            }
            else
            {
                return $"{Local}@{Domain}";
            }
        }

        public bool IsEmpty => string.IsNullOrWhiteSpace(Local) && string.IsNullOrWhiteSpace(Domain);

        protected override IEnumerable<object> GetEqualityComponents()
        {
            yield return Local;
            yield return Domain;
        }
    }
}
