﻿using KelownaSoftware.Domain.Common.Enums;
using System.Globalization;

namespace KelownaSoftware.Domain.Common.ValueObjects
{
    public class Money : ValueObject
    {
        public CurrencyCodes Currency { get; private set; }
        public decimal Amount { get; private set; }

        public Money(decimal amount, CurrencyCodes currency)
        {
            Amount = amount;
            Currency = currency;
        }

        protected override IEnumerable<object> GetEqualityComponents()
        {
            yield return Currency;
            yield return Amount;
        }

        /// <summary>
        /// Returns a string representation of the Money value consisting of the Amount and the currency.
        /// </summary>
        /// <returns>A string representation of the Money value consisting of the Amount and the currency.</returns>
        /// <remarks>The string returned is not intended for UI display.</remarks>
        public override string ToString()
        {
            return $"{Amount.ToString(CultureInfo.CurrentUICulture)}";
        }
    }
}
