﻿using KelownaSoftware.Domain.Common.Entities.Countries;
using KelownaSoftware.Domain.Common.Entities.Regions;

namespace KelownaSoftware.Domain.Common.ValueObjects.PostalAddresses
{
    public class PostalAddress : ValueObject
    {
        public PostalAddress()
        {

        }

        public PostalAddress(PostalAddress postalAddress)
        {
            Country = new CountryValueObject(postalAddress.Country);
            Region = new RegionValueObject(postalAddress.Region);
            PostalCode = postalAddress.PostalCode;
            StreetAddress = postalAddress.StreetAddress;
            StreetAddress2 = postalAddress.StreetAddress2;
            Locality = postalAddress.Locality;
        }

        public virtual RegionValueObject Region { get; set; }
        public virtual CountryValueObject Country { get; set; }

        private string _streetAddress;
        public string StreetAddress
        {
            get => _streetAddress;
            set
            {
                var temp = value?.Trim() ?? string.Empty;
                _ = temp.Length > PostalAddressConstants.STREETADDRESS_MAXLENGTH
                    ? throw new ArgumentOutOfRangeException($"Street Address length must be {PostalAddressConstants.STREETADDRESS_MAXLENGTH} or less") : 0;
                _streetAddress = temp;
            }
        }

        private string _streetAddress2;
        public string StreetAddress2
        {
            get => _streetAddress2;
            set
            {
                var temp = value?.Trim() ?? string.Empty;
                _ = temp.Length > PostalAddressConstants.STREETADDRESS2_MAXLENGTH
                    ? throw new ArgumentOutOfRangeException($"Street Address 2 length must be {PostalAddressConstants.STREETADDRESS2_MAXLENGTH} or less") : 0;
                _streetAddress2 = temp;
            }
        }

        private string _locality;
        public string Locality
        {
            get => _locality;
            set
            {
                var temp = value?.Trim() ?? string.Empty;
                _ = temp.Length > PostalAddressConstants.LOCALITY_MAXLENGTH
                    ? throw new ArgumentOutOfRangeException($"Locality length must be {PostalAddressConstants.LOCALITY_MAXLENGTH} or less") : 0;
                _locality = temp;
            }
        }

        private string _postalCode;
        public string PostalCode
        {
            get => _postalCode;
            set
            {
                //todo: add Canada/USA format validation; also cover with domain unit tests;
                var temp = value?.Trim() ?? string.Empty;
                _ = temp.Length > PostalAddressConstants.POSTALCODE_MAXLENGTH
                    ? throw new ArgumentOutOfRangeException($"Postal Code length must be {PostalAddressConstants.POSTALCODE_MAXLENGTH} or less") : 0;
                _postalCode = temp;
            }
        }

        protected override IEnumerable<object> GetEqualityComponents()
        {
            yield return StreetAddress;
            yield return StreetAddress2;
            yield return Locality;
            yield return PostalCode;
        }

        public string ToPostalAddressString()
        {
            return ToPostalAddressString(StreetAddress, StreetAddress2, Locality, Region?.Abbreviation, PostalCode, Country?.Abbreviation);
        }

        public static string ToPostalAddressString(string streetAddress, string streetAddress2, string locality, string region_Abbreviation, string postalCode, string country_Abbreviation)
        {
            // if updating, update postal-address.ts function to match
            var output = string.Empty;

            if (string.IsNullOrWhiteSpace(streetAddress2) == false && string.IsNullOrWhiteSpace(streetAddress) == false)
            {
                var cleaned = streetAddress2.Replace("#", string.Empty);
                output += $"#{cleaned} - {streetAddress}, ";
                output = output.Trim();
            }
            else if (string.IsNullOrWhiteSpace(streetAddress) == false)
            {
                output += $"{streetAddress}, ";
                output = output.Trim();
            }

            if (string.IsNullOrWhiteSpace(locality) == false)
            {
                output += $" {locality}, ";
                output = output.Trim();
            }

            if (string.IsNullOrWhiteSpace(region_Abbreviation) == false)
            {
                output += $" {region_Abbreviation}";
                output = output.Trim();
            }

            if (string.IsNullOrWhiteSpace(postalCode) == false)
            {
                output += $" {postalCode}";
                output = output.Trim();
            }

            if (string.IsNullOrWhiteSpace(country_Abbreviation) == false)
            {
                output += $" {country_Abbreviation}";
                output = output.Trim();
            }

            return output;
        }
    }
}
