﻿using KelownaSoftware.Application.Common.Interfaces.Services;
using KelownaSoftware.Application.Interfaces;
using KelownaSoftware.Application.Interfaces.Authentication;
using KelownaSoftware.Application.Interfaces.Authentication.Jwt;
using KelownaSoftware.Domain.AggregateRoots.Users;
using KelownaSoftware.Infrastructure.Services;
using KelownaSoftware.Infrastructure.Services.Authorization;
using KelownaSoftware.Persistence;
using Microsoft.AspNetCore.Identity;
using Microsoft.Extensions.DependencyInjection;
using Microsoft.Extensions.DependencyInjection.Extensions;

namespace KelownaSoftware.Infrastructure
{
    public static class DependencyInjection
    {
        public static IServiceCollection AddInfrastructure(this IServiceCollection services)
        {
            services.AddIdentityCore<User>()
                    .AddEntityFrameworkStores<KSDbContext>();

            services.TryAddScoped<UserManager<User>>();
            services.TryAddScoped<SignInManager<User>>();

            services.AddScoped<IJwtConfigurationService, JwtConfigurationService>();
            services.AddScoped<IJwtService, JwtService>();
            services.AddScoped<IRefreshTokenService, RefreshTokenService>();
            services.AddScoped<IJwtTokenValidatorService, JwtTokenValidatorService>();
            services.AddScoped<IJwtTokenHandlerService, JwtTokenHandlerService>();

            services.AddScoped<IAuthenticationService, AuthenticationService>();

            services.AddScoped<IPermissionsService, PermissionsService>();
            services.AddScoped<ICurrentUserService, CurrentUserService>();

            return services;
        }
    }
}
