﻿using KelownaSoftware.Application.AggregateRoots.Authentication;
using KelownaSoftware.Application.Interfaces.Authentication.Jwt;
using Microsoft.IdentityModel.Tokens;
using System.IdentityModel.Tokens.Jwt;
using System.Security.Claims;
using System.Text;

namespace KelownaSoftware.Infrastructure.Services.Authorization
{
    public class JwtService : IJwtService
    {
        private readonly IJwtConfigurationService _jwtConfigService;

        public JwtService(IJwtConfigurationService jwtConfigService)
        {
            _jwtConfigService = jwtConfigService;
        }

        public async Task<AccessToken> GenerateEncodedToken(IList<Claim> claims)
        {
            var securityKey = new SymmetricSecurityKey(Encoding.UTF8.GetBytes(_jwtConfigService.GetPrivateKey()));
            var credentials = new SigningCredentials(securityKey, SecurityAlgorithms.HmacSha256);

            var token = new JwtSecurityToken(_jwtConfigService.GetIssuer(),
              _jwtConfigService.GetIssuer(),
              claims,
              expires: DateTime.Now.AddMinutes(JwtTokenConstants.EXPIRY_MINUTES),
              signingCredentials: credentials);

            return new AccessToken(new JwtSecurityTokenHandler().WriteToken(token), (int)TimeSpan.FromMinutes(JwtTokenConstants.EXPIRY_MINUTES).TotalSeconds);

        }
    }
}
