﻿using KelownaSoftware.Application.Interfaces.Authentication.Jwt;
using Microsoft.IdentityModel.Tokens;
using System.Security.Claims;
using System.Text;

namespace KelownaSoftware.Infrastructure.Services.Authorization
{
    public sealed class JwtTokenValidatorService : IJwtTokenValidatorService
    {
        private readonly IJwtTokenHandlerService _jwtTokenHandler;

        public JwtTokenValidatorService(IJwtTokenHandlerService jwtTokenHandler)
        {
            _jwtTokenHandler = jwtTokenHandler;
        }

        public ClaimsPrincipal GetPrincipalFromToken(string token, string signingKey)
        {
            return _jwtTokenHandler.ValidateToken(token, new TokenValidationParameters
            {
                ValidateAudience = false,
                ValidateIssuer = false,
                ValidateIssuerSigningKey = true,
                IssuerSigningKey = new SymmetricSecurityKey(Encoding.UTF8.GetBytes(signingKey)),
                ValidateLifetime = false // we check expired tokens here
            });
        }
    }
}
