﻿using KelownaSoftware.Application.Common.Exceptions;
using KelownaSoftware.Application.Common.Interfaces.Services;
using KelownaSoftware.Application.Interfaces;
using KelownaSoftware.Domain.AggregateRoots.Users;
using Microsoft.EntityFrameworkCore;
using System.IdentityModel.Tokens.Jwt;
using System.Security.Claims;

namespace KelownaSoftware.Infrastructure.Services
{
    public class PermissionsService : IPermissionsService
    {
        private readonly IKSDbContext _context;

        public PermissionsService(IKSDbContext context)
        {
            _context = context;
        }

        public async Task<IList<Claim>> GenerateUserClaimsAsync(string userId, string email, CancellationToken cancellationToken)
        {
            var claims = new List<Claim>
            {
                new Claim(ClaimTypes.Email, email),
                new Claim(JwtRegisteredClaimNames.Jti, Guid.NewGuid().ToString()),
                new Claim(ClaimTypes.NameIdentifier, userId),
            };

            var userClaims = await _context.Users
                .AsNoTracking()
                .Where(x => x.Id == userId)
                .Select(x => x.Claims)
                .SingleOrDefaultAsync(cancellationToken)
                ?? throw new EntityNotFoundException(nameof(User), userId);

            foreach (var claim in userClaims)
            {
                claims.Add(new Claim(claim, claim));
            }

            return claims;

        }
    }
}
