﻿using KelownaSoftware.Application.Interfaces;
using KelownaSoftware.Persistence.Services;
using Microsoft.EntityFrameworkCore;
using Microsoft.Extensions.Configuration;
using Microsoft.Extensions.DependencyInjection;

namespace KelownaSoftware.Persistence
{
    public static class DependencyInjection
    {
        public static IServiceCollection AddPersistence(this IServiceCollection services, IConfiguration config)
        {
            var connectionString = config.GetConnectionString("KSDbConnection");

            services.AddDbContext<KSDbContext>(options =>
                    options.UseSqlServer(connectionString,
                    providerOptions => providerOptions
                        .EnableRetryOnFailure(25, TimeSpan.FromSeconds(10), null)
                        .CommandTimeout(300)
                    ));

            services.AddScoped<IKSDbContext, KSDbContext>();
            services.AddHostedService<MigrationKSDbHostedService>();
            services.AddHostedService<SeedKSDbHostedService>();

            return services;
        }
    }
}
