﻿using KelownaSoftware.Application.Interfaces;
using Microsoft.Extensions.DependencyInjection;
using Microsoft.Extensions.Hosting;

namespace KelownaSoftware.Persistence.Services
{
    public class SeedKSDbHostedService : IHostedService
    {
        IServiceScopeFactory _serviceScopeFactory;

        public SeedKSDbHostedService(IServiceScopeFactory serviceScopeFactory)
        {
            _serviceScopeFactory = serviceScopeFactory;
        }

        public async Task StartAsync(CancellationToken cancellationToken)
        {
            using (var serviceScope = _serviceScopeFactory.CreateScope())
            {
                var dbContext = serviceScope.ServiceProvider.GetService<IKSDbContext>();

                await dbContext.Database.EnsureCreatedAsync(cancellationToken);
                await dbContext.SeedAsync(cancellationToken);
            }
        }

        public Task StopAsync(CancellationToken cancellationToken)
        {
            //do nothing but required by the interface
            return Task.CompletedTask;
        }
    }
}
