﻿using KelownaSoftware.Application.Common.ViewModels;
using KelownaSoftware.Domain.Common.Validation;
using Microsoft.AspNetCore.Mvc;
using Microsoft.AspNetCore.Mvc.ModelBinding;

namespace KelownaSoftware.Server.Middleware
{
    // You may need to install the Microsoft.AspNetCore.Http.Abstractions package into your project
    public static class FailedValidationResponseFactory
    {
        public static ActionResult GenerateResponse(ActionContext actionContext)
        {
            var errors = GetValidationErrorsFromModelState(actionContext.ModelState);
            var result = new BaseResponse(null, errors, false);
            var response = new BadRequestObjectResult(result);
            return response;
        }

        private static List<ValidationError> GetValidationErrorsFromModelState(ModelStateDictionary modelState)
        {
            var errors = new List<ValidationError>();

            foreach (var modelItem in modelState)
            {
                // note: intentionally not handling (for now at least) the other possible error states besides Invalid: Unvalidated, Skipped
                if (modelItem.Value.ValidationState == ModelValidationState.Invalid)
                {
                    foreach (var error in modelItem.Value.Errors)
                    {
                        errors.Add(new ValidationError(modelItem.Key, error.ErrorMessage));
                    }
                }
            }

            return errors;
        }
    }
}
